# # Chapter 2
#
# This demonstrates integrating source code into an mdbook. All comments are treated as **Markdown**.
#
# Note that this file doesn't need to be named `chapter_x.py` -- *any* name is acceptable.
print("Source code is rendered as expected.")
