# .. Copyright (C) 2012-2020 Bryan A. Jones.
#
#   This file is part of the CodeChat System.
#
#   The CodeChat System is free software: you can redistribute it and/or
#   modify it under the terms of the GNU General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   The CodeChat System is distributed in the hope that it will be
#   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
#   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#   General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with the CodeChat System.  If not, see
#   <http://www.gnu.org/licenses/>.
#
# *****************************
# |docname| - Module definition
# *****************************
# Give the version number, which is read by `../setup.py` during packaging.
# This is chosen following the `version convention
# <http://packaging.python.org/en/latest/tutorial.html#version>`_.
__version__ = "0.2.1"

# Contents:
#
# .. toctree::
#   :maxdepth: 2
#
#   server.py
#   render_manager.py
#   renderer.py
#   __main__.py
#   constants.py
